

import stamp.core.*;
import com.micromegacorp.math.hardware.*;

class TestSHT11
{
  static Float32 m_fValue = new Float32();
  static Float32 m_fValue2 = new Float32();
  static Float32 m_fValue3 = new Float32();
  static Float32 m_fValue4 = new Float32();

  static Float32 m_fTemperature = new Float32();

  static Timer m_Timer = new Timer();
  static StringBuffer m_StringBuffer = new StringBuffer();

  static final int DATA_PIN = CPU.pin4;
  static final int CLOCK_PIN = CPU.pin5;

  static final int shtTemp = 0x0003;
  static final int shtHumi = 0x0005;
  static final int shtStatW = 0x0006;
  static final int shtStatR = 0x0007;
  static final int shtReset = 0x001E;

  static int m_nTries = 0; // The number of tries we've made to retrieve the temperature from the SHT11

  public TestSHT11()
  {
    m_nTries = 0;
  }

  public static void main()
  {

    // reset the uM-FPU and get version
    if (!Fpu.reset()) {
      System.out.println("uM-FPU not responding.");
      return;
    }
    System.out.println(Fpu.version());


    shtConnectionReset();
    while ( true )
    {
      // If we tried to read the temperature 3 times but we're not able to, exit the program
      if (m_nTries >= 3 )
        break;

      MeasureTemp();         // Measure the temperature
      MeasureHumidity();

      CPU.delay( 5000 );  // Long delay

      System.out.print('\u0010');  // Clears the terminal
    }
  }

  // This function makes the SHT11 is ready with the measurement or ~1/4 seconds. According to the
  // SHT11 documentation, the measurement can take up to 210 ms
  static void WaitForMeasurement()
  {
    CPU.writePin( DATA_PIN, true );
    CPU.setInput( DATA_PIN );

    m_Timer.mark();

    boolean bValue = true;
    while (!m_Timer.timeout(250))
    {
      bValue = CPU.readPin( DATA_PIN );
      if ( !bValue )
        break;
    }
  }


  // connection reset: 9 clock cyles with ShtData high, then start sequence
  // Function initializes the SHT11 chip
  // generates SHT11 "connection reset" sequence
  //           ______________________________________________
  // ShtData
  //              ___     ___     ___     ____     ___     __.
  // Clock    ___| 1 |___| 2 |___| 3 |___|4-8 |___| 9 |___|
  static void shtConnectionReset()
  {
    CPU.writePin( DATA_PIN, true );
    CPU.setInput( DATA_PIN );
    CPU.writePin( CLOCK_PIN, false );

    for ( int i = 0; i < 9; i++ )
    {
      // 0 - 8 = 9
      CPU.writePin( CLOCK_PIN, true );
      CPU.writePin( CLOCK_PIN, false );
    }

    CPU.writePin( CLOCK_PIN, true );
  }

  //                         ___
  // ShtData        |_______|
  //                _     _
  // Clock           |___|
  // generates SHT11 "start" sequence
  static void shtTransmissionStart()
  {
    // ensure we have the correct values
    CPU.writePin( DATA_PIN, true );
    CPU.writePin( CLOCK_PIN, true );


    CPU.writePin( DATA_PIN, false );
    CPU.writePin( CLOCK_PIN, false );
    CPU.writePin( CLOCK_PIN, true );
    CPU.writePin( DATA_PIN, true );
    CPU.writePin( CLOCK_PIN, false );
    CPU.setInput( DATA_PIN );
  }
  static void MeasureHumidity()
  {
    shtTransmissionStart(); // Alert device

    shtWriteByte( shtHumi ); // send humidity command
    WaitForMeasurement();    // wait for measurement

    // Get the humidity
    int nHumidity = shtReadByte( true ) << 8; // Get the MSB
    nHumidity |= shtReadByte( false );

    // Linearize humidity:
    // m_fValue = (soRH * 0.0405) - (soRH * 0.002 * soRH * 0.0014) - 4
    m_fValue.set( nHumidity );
    m_fValue2.set( "0.0405" );
    m_fValue.multiply( m_fValue2 );

    m_fValue2.set( nHumidity );
    m_fValue3.set( "0.002" );

    m_fValue2.multiply( m_fValue3 );
    m_fValue2.multiply( nHumidity );
    m_fValue3.set( "0.0014" );
    m_fValue2.multiply( m_fValue3 );
    m_fValue2.subtract( 4 );
    m_fValue.subtract( m_fValue2 );

    // temperature compensated humidity
    // m_fValue2 = (tc - 25) * (soRH * 0.00008 + 0.01) + m_fValue
    m_fValue2.set( m_fTemperature );
    m_fValue2.subtract( 25 );

    m_fValue3.set( nHumidity );
    m_fValue4.set("0.00008");
    m_fValue3.multiply( m_fValue4 );
    m_fValue4.set("0.01");
    m_fValue3.add( m_fValue4 );

    m_fValue2.multiply( m_fValue3 );
    m_fValue2.add( m_fValue);

    // m_fValue2 is "rhTrue"
    // m_fValue is "rhLin"
    System.out.println( m_fValue.toString() );
    System.out.println( m_fValue2.toString() );
  }

  static void MeasureTemp()
  {
    shtTransmissionStart();   // Alert device
    shtWriteByte( shtTemp );  // send temperature command
    WaitForMeasurement();     // wait for measurement

    // Get the temperature
    int nTemperature = shtReadByte( true ) << 8; // Get the MSB
    nTemperature |= shtReadByte( false );

    boolean bNegative = nTemperature > 32767;
    if ( bNegative )
    {
      // Something is wrong with the communications...
      // since we're getting in a negative value.
      // Reset the connection to SHT11
      shtConnectionReset();
      MeasureTemp();

      m_nTries++; // Increase the number of times we failed to read the temperature
      System.out.println("Warning...could not read the temperature...");
    }
    else
    {
      m_nTries = 0; // Reset the number of tries...
    }

    // Calculate the temperature celsius
    m_fValue.set( nTemperature );
    m_fValue2.set("0.01");
    m_fValue.multiply( m_fValue2 );
    m_fValue.subtract( 40 );
    m_fValue.multiply( 10 );
    m_fValue2.set( m_fValue );
    int nTemp = Float32Math.round( m_fValue2 );

    m_fTemperature.set( nTemp );
    m_fTemperature.divide( 10 );
    String str = m_fTemperature.toString();
    m_StringBuffer.clear();

    int nIndex = str.indexOf(".");
    if ( nIndex > 0 ) // The "." could be found
    {
      // Copy only the string until "." + one character
      for ( int i = 0; i <= nIndex + 1; i++ )
        m_StringBuffer.append( str.charAt( i ) );
    }
    else
    {
        m_StringBuffer.append( str );
    }

    System.out.println( m_StringBuffer.toString() );
  }

  static int shtReadByte( boolean bSendAck )
  {
    CPU.writePin( CLOCK_PIN, false );   // Prepare for shiftIn command
    int nData = CPU.shiftIn( DATA_PIN, CLOCK_PIN, 8, CPU.PRE_CLOCK_MSB );
    CPU.writePin( CLOCK_PIN, false );

    if ( bSendAck )
    {
      // Send the acknowledge bit
      CPU.writePin( DATA_PIN, false );
      CPU.writePin( CLOCK_PIN, false );
      CPU.writePin( CLOCK_PIN, true );
      CPU.writePin( CLOCK_PIN, false );
      CPU.setInput( DATA_PIN );
    }
    else
    {
      // Skip the acknowledge bit
      CPU.shiftOut( DATA_PIN, CLOCK_PIN, 1, CPU.SHIFT_LSB, 1 );
    }
    return ( nData );
  }
  // Function writes 8 bits to the SHT11
  static void shtWriteByte( int data )
  {
    CPU.writePin( CLOCK_PIN, false );
    data = data << 8;
    CPU.shiftOut( DATA_PIN, CLOCK_PIN, 8, CPU.SHIFT_MSB, data );

    CPU.writePin( CLOCK_PIN, false );   // Prepare for shiftIn command
    CPU.shiftIn( DATA_PIN, CLOCK_PIN, 1, CPU.POST_CLOCK_LSB ); // Get the acknowledge bit...
  }
}